@@ -2,7 +2,7 @@
  * wince.h: private WinCE interface descriptor
  *****************************************************************************
  * Copyright (C) 1999-2004 the VideoLAN team
- * $Id: wince.h 11988 2005-08-03 19:01:44Z courmisch $
+ * $Id: e7ccf3b8f8014a2e7cebbe9b99ff2d041408baee $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *          Marodon Cedric <cedric_marodon@yahoo.fr>
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifndef WINCE_RESOURCE
@@ -40,11 +40,12 @@
 #endif
 
 #include "vlc_keys.h"
+#include <vlc_messages.h>
 
 #include <stdio.h>
 #include <string>
 #include <vector>
-using namespace std; 
+using namespace std;
 
 class CBaseWindow;
 class MenuItemExt;
@@ -59,7 +60,7 @@ struct intf_sys_t
     CBaseWindow         *p_window;
 
     /* special actions */
-    vlc_bool_t          b_playing;
+    bool          b_playing;
 
     /* The input thread */
     input_thread_t *    p_input;
@@ -67,10 +68,9 @@ struct intf_sys_t
     /* The slider */
     int                 i_slider_pos;                     /* slider position */
     int                 i_slider_oldpos;                /* previous position */
-    vlc_bool_t          b_slider_free;                      /* slider status */
+    bool          b_slider_free;                      /* slider status */
+
 
-    /* The messages window */
-    msg_subscription_t* p_sub;                  /* message bank subscription */
 
     /* Playlist management */
     int                 i_playing;                 /* playlist selected item */
@@ -86,6 +86,8 @@ struct intf_sys_t
     vector<MenuItemExt*> *p_settings_menu;
 
     VideoWindow          *p_video_window;
+
+    HANDLE   thread_ready;
 };
 
 /*****************************************************************************
@@ -187,7 +189,7 @@ private:
     HMENU menu_audio;
     HMENU menu_navigation;
 
-    vlc_bool_t b_volume_hold;
+    bool b_volume_hold;
 };
 
 /* File Info */
@@ -203,7 +205,7 @@ public:
 protected:
 
     HWND hwnd_fileinfo;                 // handle to fileinfo window
-    HWND hwndTV;                                // handle to tree-view control 
+    HWND hwndTV;                                // handle to tree-view control
 
     TCHAR szFileInfoClassName[100];     // Main window class name
     TCHAR szFileInfoTitle[100];         // Main window name
@@ -212,22 +214,31 @@ protected:
     BOOL CreateTreeView( HWND );
 };
 
+struct msg_cb_data_t
+{
+    Messages *self;
+};
+
 /* Messages */
 class Messages : public CBaseWindow
 {
 public:
     /* Constructor */
     Messages( intf_thread_t *, CBaseWindow *, HINSTANCE );
-    virtual ~Messages(){};
+     ~Messages();
 
-    void UpdateLog(void);
+    static void sinkMessage (msg_cb_data_t *, msg_item_t *, unsigned);
+    void sinkMessage (msg_item_t *item, unsigned);
 
 protected:
 
     virtual LRESULT WndProc( HWND, UINT, WPARAM, LPARAM );
 
+    /* The messages window */
+    msg_subscription_t* sub;                  /* message bank subscription */
+    msg_cb_data_t *cb_data;
     HWND hListView;
-    vlc_bool_t b_verbose;
+    bool b_verbose;
 };
 
 /* ItemInfo Dialog */
@@ -432,7 +443,7 @@ public:
     /* Constructor */
     Timer( intf_thread_t *p_intf, HWND hwnd, Interface *_p_main_interface );
     virtual ~Timer();
-    void Notify( void ); 
+    void Notify( void );
 
 private:
     intf_thread_t *p_intf;
@@ -448,11 +459,11 @@ void RefreshAudioMenu( intf_thread_t *_p_intf, HMENU hMenu );
 void RefreshVideoMenu( intf_thread_t *_p_intf, HMENU hMenu );
 void RefreshNavigMenu( intf_thread_t *_p_intf, HMENU hMenu );
 void RefreshMenu( intf_thread_t *, vector<MenuItemExt*> *, HMENU, int,
-                  char **, int *, int );
+                  char **, vlc_object_t **, int );
 int wce_GetMenuItemCount( HMENU );
 void CreateMenuItem( intf_thread_t *, vector<MenuItemExt*> *, HMENU, char *,
                      vlc_object_t *, int * );
-HMENU CreateChoicesMenu( intf_thread_t *, vector<MenuItemExt*> *, char *, 
+HMENU CreateChoicesMenu( intf_thread_t *, vector<MenuItemExt*> *, char *,
                          vlc_object_t *, int * );
 void OnMenuEvent( intf_thread_t *, int );
 
@@ -465,7 +476,7 @@ class MenuItemExt
 public:
     /* Constructor */
     MenuItemExt( intf_thread_t *_p_intf, int _id, char *_psz_var,
-                 int _i_object_id, vlc_value_t _val, int _i_val_type );
+                 vlc_object_t * p_object, vlc_value_t _val, int _i_val_type );
 
     virtual ~MenuItemExt();
 
@@ -475,7 +486,7 @@ public:
     intf_thread_t *p_intf;
     char *psz_var;
     int  i_val_type;
-    int  i_object_id;
+    vlc_object_t * p_object;
     vlc_value_t val;
 
 private:
